import React, { createContext, useRef, useState } from 'react';

export const UserContext = createContext({
    username: "",
    permission: [] as string[],
    setUsername: (s: string) => { },
    setPermission: (permission: string[]) => { }
});

type LoginLocalStorage = {
    username: string,
    permission: string[],
    refreshExpired: number
};

let localObj: LoginLocalStorage | null = null;

export const getInitialValue = async () => {
    if (localObj === null) {
        const storagestr = window.localStorage.getItem("UserLoginContext") as string | null;
        const storage = storagestr !== null ? JSON.parse(storagestr) as LoginLocalStorage | null : null;
        localObj = storage;
    }   
    if (localObj !== null && localObj.refreshExpired > Math.floor(Date.now() / 1000)) {
        return {
            username: localObj.username,
            permission: localObj.permission,
        }
    }
    const res = await fetch('/user/refresh', {
        method: 'POST',
    });
    if (res.status !== 200) throw new Error("Maybe Network Error")
    const r = await res.json() as LoginLocalStorage & { refresh: boolean };
    if (r.refresh) {
        localObj = {
            username: r.username,
            permission: r.permission,
            refreshExpired: r.refreshExpired
        }
    }
    else {
        localObj = {
            refreshExpired: 0,
            username: "",
            permission: r.permission
        }
    }
    window.localStorage.setItem("UserLoginContext", JSON.stringify(localObj));
    return {
        username: r.username,
        permission: r.permission
    }
}
export const doLogout = async () => {
    const req = await fetch('/user/logout', {
        method: 'POST'
    });
    try {
        const res = await req.json();
        localObj = {
            refreshExpired: 0,
            username: "",
            permission: res["permission"]
        }
        window.localStorage.setItem("UserLoginContext", JSON.stringify(localObj));
        return {
            username: localObj.username,
            permission: localObj.permission,
        }
    } catch (error) {
        console.error(`Server Error ${error}`);
        return {
            username: "",
            permission: [],
        }
    }
}
export const doLogin = async (userLoginInfo:{
    username:string,
    password:string,
}): Promise<string|LoginLocalStorage>=>{
    const res = await fetch('/user/login',{
        method:'POST',
        body:JSON.stringify(userLoginInfo),
        headers:{"content-type":"application/json"}
    });
    const b = await res.json();
    if(res.status !== 200){
        return b.detail as string;
    }
    localObj = b;
    window.localStorage.setItem("UserLoginContext", JSON.stringify(localObj));
    return b;
}